addpath 1SpectralClustering
addpath Ncut_9

datanames = {...
    'Amazon', ...
    'Iris', ...
    'Votes', ...
    'ORL', ...
    'PIE', ...
    'YaleB', ...
    'COIL-20', ...
    'Isolet', ...
    'Mfeat', ...
    'Webkb4', ...
    '7sectors', ...
    'USPS', ...
    'PenDigits', ...
    'LetReco', ...
    'MNIST', ...
    };
nd = length(datanames);

methods = {...
    'Ncut', ...
    'PNMF', ...
    'NSC', ...
    'ONMF', ...
    'PLSI', ...
    'LSD', ...
    '1-Spec', ...
    'DCD', ...
    };
nm = length(methods);

max_iter = 10000;
nSeed = 0;
alphas = [1.2 2 5];
na = length(alphas);

purity = zeros(nd,nm);

for di=1:nd
    dataname = datanames{di};
    fprintf('%-11s', dataname);
    
    [A,C,nc] = GetClusteringData(dataname);
    W0 = ncutW(A, nc, nSeed);
    [~,idx] = max(W0,[],2);
    purity(di,1) = ComputeClusterPurity(idx, C, nc);
    fprintf('\tNcut:%.2f', purity(di,1));
    
    W0 = W0 + 0.2;
    for mi=2:nm
        method = methods{mi};
        fprintf('\t%s:', method);
        switch method
            case 'PNMF'
                W = pnmf(A, max_iter, W0);
            case 'NSC'
                W = nsc(A, max_iter, W0);
            case 'ONMF'
                W = onmf(A, max_iter, W0);
            case 'PLSI'
                [W,s] = plsi(A, max_iter, W0);
                W = bsxfun(@times, W, s');
                W = bsxfun(@rdivide, W, sum(W,2)+eps);
            case 'LSD'
                W = lsd(A, max_iter, W0);
            case '1-Spec'
                [clusters,~,~] = OneSpectralClustering(A, 'rcc', nc, 0, 0, 0);
                W = full(clabel2dataclasses(clusters(:,end),nc));
            case 'DCD'
                cross_entropy = zeros(na,1);
                Ws = cell(na,1);
                for ai=1:na
                    alpha = alphas(ai);
                    Winit = dcd(A, nc, alpha, max_iter, W0, nSeed);
                    [Ws{ai},cross_entropy(ai)] = dcd(A, nc, 1, max_iter, Winit+0.2, nSeed);
                end
                [~,ind] = min(cross_entropy);
                W = Ws{ind};
        end
        [~,idx] = max(W,[],2);
        purity(di,mi) = ComputeClusterPurity(idx, C, nc);
        
        fprintf('%.2f', purity(di,mi));
        
        save('results_clustering_test.mat', ...
            'datanames', 'methods', ...
            'max_iter', 'nSeed', 'alphas', 'purity');
    end
    fprintf('\n');
end
